@@ -1,4 +1,3 @@
-#! /usr/bin/env python3
 """genmixdb
 
 usage: genmixdb [-hv] CONFIG...
@@ -120,6 +119,7 @@ will find all .wav files in the specified directories and process them as target
 import json
 import re
 from copy import deepcopy
+from functools import partial
 from glob import glob
 from numbers import Number
 from os import listdir
@@ -137,8 +137,8 @@ import numpy as np
 import sox
 import yaml
 from docopt import docopt
+from p_tqdm import p_map
 from pyaaware import FeatureGenerator
-from tqdm import tqdm
 
 import sonusai
 from sonusai import create_file_handler
@@ -146,19 +146,20 @@ from sonusai import initial_log_messages
 from sonusai import logger
 from sonusai import update_console_handler
 from sonusai.mixture import apply_augmentation
+from sonusai.mixture import estimate_audio_length
 from sonusai.mixture import generate_truth
+from sonusai.mixture import get_class_count
+from sonusai.mixture import get_class_weights_threshold
 from sonusai.mixture import get_config_from_file
 from sonusai.mixture import get_next_noise
+from sonusai.mixture import get_total_class_count
 from sonusai.mixture import read_audio
 from sonusai.utils import expandvars
 from sonusai.utils import human_readable_size
 from sonusai.utils import trim_docstring
 
 
-def genmixdb(config: dict, verbose: bool = False, show_progress: bool = False) -> dict:
-    update_console_handler(verbose)
-    initial_log_messages('genmixdb')
-
+def genmixdb(config: dict, show_progress: bool = False) -> dict:
     required_keys = [
         'class_labels',
         'class_weights_threshold',
@@ -195,14 +196,14 @@ def genmixdb(config: dict, verbose: bool = False, show_progress: bool = False) -
         logger.error('Canceled due to no targets')
         exit()
 
-    logger.debug('Expanded list of targets:')
+    logger.debug('List of targets:')
     logger.debug(yaml.dump([sub['name'] for sub in targets], default_flow_style=False))
 
     noises = get_input_files(config['noises'])
     if len(noises) == 0:
         logger.warning('Did not find any noises; using default noise: {}'.format(sonusai.mixture.default_noise))
         noises = get_input_files([sonusai.mixture.default_noise], silent=True)
-    logger.debug('Expanded list of noises:')
+    logger.debug('List of noises:')
     logger.debug(yaml.dump([sub['name'] for sub in noises], default_flow_style=False))
 
     target_augmentations = get_augmentations(config['target_augmentations'], target=True)
@@ -219,21 +220,16 @@ def genmixdb(config: dict, verbose: bool = False, show_progress: bool = False) -
     logger.debug('Expanded list of noise augmentations:')
     logger.debug(expanded_noise_augmentations)
 
-    total_combinations = len(targets) * len(noises) * len(target_augmentations) * len(noise_augmentations)
+    noise_sets = len(noises) * len(noise_augmentations)
+    target_sets = len(targets) * len(target_augmentations)
+    total_mixtures = noise_sets * target_sets
     logger.info('')
-    logger.info('Found {} combinations to process'.format(total_combinations))
+    logger.info('Found {} mixtures to process'.format(total_mixtures))
 
-    # TODO: check calculation of estimated duration; this is probably not quite correct
-    total_duration = len(noises) * len(noise_augmentations) * sum([sub['duration'] for sub in targets]) * len(
-        target_augmentations)
+    total_duration = len(noises) * len(noise_augmentations) * \
+                     sum([sub['duration'] for sub in targets]) * len(target_augmentations)
 
-    class_weights_threshold = config['class_weights_threshold']
-    if not isinstance(class_weights_threshold, list):
-        class_weights_threshold = np.repeat(class_weights_threshold, config['num_classes'])
-
-    if len(class_weights_threshold) != config['num_classes']:
-        logger.error('invalid class_weights_threshold length: {}'.format(len(class_weights_threshold)))
-        exit()
+    class_weights_threshold = get_class_weights_threshold(config)
 
     if config['truth_mode'] not in ['normal', 'mutex']:
         logger.error('invalid truth_mode: {}'.format(config['truth_mode']))
@@ -269,16 +265,22 @@ def genmixdb(config: dict, verbose: bool = False, show_progress: bool = False) -
     # Truth
     total_size += total_features * sonusai.mixture.float_bytes
 
+    mixture_bytes = total_size
+    truth_t_bytes = total_size * config['num_classes'] * 2
+    feature_bytes = total_size / feature_samples * stride * num_bands * 2
+    truth_f_bytes = total_size / feature_samples * config['num_classes'] * 2
     logger.info('')
-    logger.info('Estimated duration:   {}'.format(strftime('%H:%M:%S', gmtime(total_duration))))
-    logger.info('Estimated size:       {}'.format(human_readable_size(total_size, 0)))
-    logger.info('Feature shape:        {} x {} ({} total params)'.format(stride, num_bands,
-                                                                         stride * num_bands))
+    logger.info('Estimated duration:   {} (HH:MM:SS)'.format(strftime('%H:%M:%S', gmtime(total_duration))))
+    logger.info('Estimated sizes:')
+    logger.info(' mixture:             {}'.format(human_readable_size(mixture_bytes, 1)))
+    logger.info(' truth_t:             {}'.format(human_readable_size(truth_t_bytes, 1)))
+    logger.info(' feature:             {}'.format(human_readable_size(feature_bytes, 1)))
+    logger.info(' truth_f:             {}'.format(human_readable_size(truth_f_bytes, 1)))
+    logger.info('Feature shape:        {} x {} ({} total params)'.format(stride, num_bands, stride * num_bands))
     logger.info('Feature samples:      {} samples ({} ms)'.format(feature_samples, feature_ms))
     logger.info('Feature step samples: {} samples ({} ms)'.format(feature_step_samples, feature_step_ms))
-    logger.info('')
 
-    mixture_database = {
+    mixdb = {
         'class_count':              [],
         'class_labels':             config['class_labels'],
         'class_weights_threshold':  list(class_weights_threshold),
@@ -296,130 +298,91 @@ def genmixdb(config: dict, verbose: bool = False, show_progress: bool = False) -
         'truth_reduction_function': config['truth_reduction_function'],
     }
 
-    class_count = [0] * config['num_classes']
-    total_samples = 0
-    mixtures = []
-
     # Read in all audio data beforehand to avoid reading it multiple times in the loop
+    logger.debug('Reading noise audio')
     raw_noise_audio = []
     for noise in noises:
         raw_noise_audio.append(read_audio(name=noise['name'], dither=config['dither']))
 
+    logger.debug('Reading target audio')
     raw_target_audio = []
     for target in targets:
         raw_target_audio.append(read_audio(name=target['name'], dither=config['dither']))
 
-    with tqdm(desc='Processing', total=total_combinations, disable=(not show_progress)) as progress_bar:
-        for noise_index, noise in enumerate(noises):
-            for noise_augmentation_index, noise_augmentation in enumerate(noise_augmentations):
-                augmented_noise_audio = apply_augmentation(audio_in=raw_noise_audio[noise_index],
-                                                           augmentation=noise_augmentation,
-                                                           length_common_denominator=1,
-                                                           dither=config['dither'])
-                noise_offset = 0
-                for target_index, target in enumerate(targets):
-                    for target_augmentation_index, target_augmentation in enumerate(target_augmentations):
-                        if not isinstance(target['truth_index'], list):
-                            target['truth_index'] = [target['truth_index']]
+    # First pass to get indices and offsets
+    mixtures = [[] for n in range(noise_sets)]
+    n_id = 0
+    for noise_index, noise in enumerate(noises):
+        for noise_augmentation_index, noise_augmentation in enumerate(noise_augmentations):
+            mixtures[n_id] = [[] for n in range(target_sets)]
+            t_id = 0
+            noise_offset = 0
+            noise_length = estimate_audio_length(audio_in=raw_noise_audio[noise_index],
+                                                 augmentation=noise_augmentation,
+                                                 length_common_denominator=1)
+            for target_index, target in enumerate(targets):
+                for target_augmentation_index, target_augmentation in enumerate(target_augmentations):
+                    if not isinstance(target['truth_index'], list):
+                        target['truth_index'] = [target['truth_index']]
 
-                        if any(i > max_class for i in target['truth_index']):
-                            logger.error('invalid truth_index')
-                            exit()
+                    if any(i > max_class for i in target['truth_index']):
+                        logger.error('invalid truth_index')
+                        exit()
 
-                        mixture_record = {'target_file_index':         target_index,
-                                          'noise_file_index':          noise_index,
-                                          'noise_offset':              noise_offset,
-                                          'target_augmentation_index': target_augmentation_index,
-                                          'noise_augmentation_index':  noise_augmentation_index}
+                    mixtures[n_id][t_id] = {'target_file_index':         target_index,
+                                            'noise_file_index':          noise_index,
+                                            'noise_offset':              noise_offset,
+                                            'target_augmentation_index': target_augmentation_index,
+                                            'noise_augmentation_index':  noise_augmentation_index}
+                    t_id += 1
 
-                        augmented_target_audio = apply_augmentation(audio_in=raw_target_audio[target_index],
-                                                                    augmentation=target_augmentation,
-                                                                    length_common_denominator=feature_step_samples,
-                                                                    dither=config['dither'])
+                    target_length = estimate_audio_length(audio_in=raw_target_audio[target_index],
+                                                          augmentation=target_augmentation,
+                                                          length_common_denominator=feature_step_samples)
+                    noise_offset = int((noise_offset + target_length) % noise_length)
 
-                        noise_segment, noise_offset = get_next_noise(offset_in=noise_offset,
-                                                                     length=len(augmented_target_audio),
-                                                                     audio_in=augmented_noise_audio)
+            n_id += 1
 
-                        # target_gain is the sum of 'gain', noise overflow adjustment, and mixture
-                        # overflow adjustment. It is used to return the target audio to its normalized
-                        # level when calculating truth (unless target gain is set to zero, in which
-                        # case the truth will be all zeros).
-                        if 'gain' in target_augmentation:
-                            target_gain = 10 ** (target_augmentation['gain'] / 20)
-                        else:
-                            target_gain = 1
+    # Second pass to fill in the details
+    for n_id in range(len(mixtures)):
+        augmented_noise_audio = apply_augmentation(
+            audio_in=raw_noise_audio[mixtures[n_id][0]['noise_file_index']],
+            augmentation=noise_augmentations[mixtures[n_id][0]['noise_augmentation_index']],
+            length_common_denominator=1,
+            dither=config['dither'])
 
-                        if 'snr' in target_augmentation:
-                            if target_augmentation['snr'] < -96:
-                                # Special case for zeroing out target data
-                                mixture_record['target_snr_gain'] = 0
-                                mixture_record['noise_snr_gain'] = 1
-                                target_gain = 0
-                            else:
-                                target_energy = np.mean(np.square(augmented_target_audio.astype(np.single)))
-                                noise_energy = np.mean(np.square(noise_segment.astype(np.single)))
-                                noise_gain = np.sqrt(target_energy / noise_energy) / 10 ** (
-                                        target_augmentation['snr'] / 20)
+        desc = 'Processing {} of {}'.format(n_id + 1, len(mixtures))
+        mixtures[n_id] = p_map(partial(process_mixture,
+                                       config=config,
+                                       targets=targets,
+                                       target_augmentations=target_augmentations,
+                                       raw_target_audio=raw_target_audio,
+                                       augmented_noise_audio=augmented_noise_audio,
+                                       feature_step_samples=feature_step_samples,
+                                       truth_mutex=truth_mutex,
+                                       class_weights_threshold=class_weights_threshold),
+                               mixtures[n_id],
+                               desc=desc)
 
-                                # Check for noise_gain > 1 to avoid clipping
-                                if noise_gain > 1:
-                                    mixture_record['target_snr_gain'] = 1 / noise_gain
-                                    mixture_record['noise_snr_gain'] = 1
-                                    target_gain = target_gain / noise_gain
-                                else:
-                                    mixture_record['target_snr_gain'] = 1
-                                    mixture_record['noise_snr_gain'] = noise_gain
-
-                        truth_config = deepcopy(target['truth_config'])
-                        truth_config['index'] = target['truth_index']
-                        truth_config['frame_size'] = config['frame_size']
-                        truth_config['num_classes'] = config['num_classes']
-                        truth_config['mutex'] = truth_mutex
-
-                        if target_gain == 0:
-                            truth = np.zeros((config['num_classes'], len(augmented_target_audio)), dtype=np.int16)
-                        else:
-                            truth = generate_truth(
-                                audio=(augmented_target_audio.astype(np.single) / target_gain).astype(np.int16),
-                                function=target['truth_function'],
-                                config=truth_config)
-
-                        for c in target['truth_index']:
-                            class_count[c - 1] += np.sum(truth[c - 1, :] >= class_weights_threshold[c - 1])
-
-                        # Check for clipping in mixture
-                        mixture_audio = np.single(augmented_target_audio) * mixture_record['target_snr_gain'] + \
-                                        np.single(noise_segment) * mixture_record['noise_snr_gain']
-
-                        if any(abs(mixture_audio) >= 32768):
-                            # Clipping occurred; lower gains to bring audio within int16 bounds
-                            gain_adjustment = 32760 / max(abs(mixture_audio))
-                            mixture_record['target_snr_gain'] *= gain_adjustment
-                            mixture_record['noise_snr_gain'] *= gain_adjustment
-                            target_gain *= gain_adjustment
-
-                        target_len = len(augmented_target_audio)
-                        total_samples += target_len
-
-                        mixture_record['samples'] = target_len
-                        mixture_record['target_gain'] = target_gain
-                        mixtures.append(mixture_record)
-
-                        progress_bar.update()
-
-    mixture_database['class_count'] = [int(i) for i in class_count]
-    mixture_database['mixtures'] = mixtures
+    # Flatten mixtures
+    mixdb['mixtures'] = [item for sublist in mixtures for item in sublist]
+    mixdb['class_count'] = get_total_class_count(mixdb)
 
+    total_samples = sum([sub['samples'] for sub in mixdb['mixtures']])
+    total_duration = total_samples / sonusai.mixture.sample_rate
+    mixture_bytes = total_samples * sonusai.mixture.bit_depth / 8
+    truth_t_bytes = total_samples * config['num_classes'] * 4
+    feature_bytes = total_samples / feature_samples * stride * num_bands * 4
+    truth_f_bytes = total_samples / feature_samples * config['num_classes'] * 4
     logger.info('')
-    logger.info(
-        'Actual duration: {} (HH:MM:SS)'.format(
-            strftime('%H:%M:%S', gmtime(total_samples / sonusai.mixture.sample_rate))))
-    logger.info('Actual size:     {} (feature and truth_f)'.format(
-        human_readable_size(total_samples * sonusai.mixture.bit_depth / 8)))
-    logger.info('Finished')
+    logger.info('Actual duration: {} (HH:MM:SS)'.format(strftime('%H:%M:%S', gmtime(total_duration))))
+    logger.info('Actual sizes:')
+    logger.info(' mixture:        {}'.format(human_readable_size(mixture_bytes, 1)))
+    logger.info(' truth_t:        {}'.format(human_readable_size(truth_t_bytes, 1)))
+    logger.info(' feature:        {}'.format(human_readable_size(feature_bytes, 1)))
+    logger.info(' truth_f:        {}'.format(human_readable_size(truth_f_bytes, 1)))
 
-    return mixture_database
+    return mixdb
 
 
 def get_input_files(records, cfg=None, silent=False):
@@ -501,10 +464,9 @@ def append_input_files(files: list, in_record, cfg: dict, tokens=None):
                 else:
                     sox.file_info.validate_input_file(name)
                     duration = sox.file_info.duration(name)
-                    original_name = name
                     for key, value in tokens.items():
-                        original_name.replace(value, key)
-                    entry = {'name': name, 'duration': duration, 'original_name': original_name}
+                        name = name.replace(value, '${}'.format(key))
+                    entry = {'name': name, 'duration': duration}
                     entry.update(cfg)
                     files.append(entry)
             except Exception as e:
@@ -637,29 +599,128 @@ def get_base_name(config: dict, config_name: str) -> str:
         exit()
 
 
+def process_mixture(mixture: dict,
+                    config: dict,
+                    targets: list,
+                    target_augmentations: list,
+                    raw_target_audio: list,
+                    augmented_noise_audio: np.ndarray,
+                    feature_step_samples: int,
+                    truth_mutex: int,
+                    class_weights_threshold: list) -> dict:
+    target_index = mixture['target_file_index']
+    noise_offset = mixture['noise_offset']
+    target_augmentation_index = mixture['target_augmentation_index']
+
+    target = targets[target_index]
+
+    target_augmentation = target_augmentations[target_augmentation_index]
+
+    augmented_target_audio = apply_augmentation(audio_in=raw_target_audio[target_index],
+                                                augmentation=target_augmentation,
+                                                length_common_denominator=feature_step_samples,
+                                                dither=config['dither'])
+
+    noise_segment, _ = get_next_noise(offset_in=noise_offset,
+                                      length=len(augmented_target_audio),
+                                      audio_in=augmented_noise_audio)
+
+    # target_gain is the sum of 'gain', noise overflow adjustment, and mixture
+    # overflow adjustment. It is used to return the target audio to its normalized
+    # level when calculating truth (unless target gain is set to zero, in which
+    # case the truth will be all zeros).
+    if 'gain' in target_augmentation:
+        target_gain = 10 ** (target_augmentation['gain'] / 20)
+    else:
+        target_gain = 1
+
+    if 'snr' in target_augmentation:
+        if target_augmentation['snr'] < -96:
+            # Special case for zeroing out target data
+            mixture['target_snr_gain'] = 0
+            mixture['noise_snr_gain'] = 1
+            target_gain = 0
+        else:
+            target_energy = np.mean(np.square(np.single(augmented_target_audio)))
+            noise_energy = np.mean(np.square(np.single(noise_segment)))
+            noise_gain = np.sqrt(target_energy / noise_energy) / 10 ** (
+                    target_augmentation['snr'] / 20)
+
+            # Check for noise_gain > 1 to avoid clipping
+            if noise_gain > 1:
+                mixture['target_snr_gain'] = 1 / noise_gain
+                mixture['noise_snr_gain'] = 1
+                target_gain = target_gain / noise_gain
+            else:
+                mixture['target_snr_gain'] = 1
+                mixture['noise_snr_gain'] = noise_gain
+
+    truth_config = deepcopy(target['truth_config'])
+    truth_config['index'] = target['truth_index']
+    truth_config['frame_size'] = config['frame_size']
+    truth_config['num_classes'] = config['num_classes']
+    truth_config['mutex'] = truth_mutex
+
+    # Check for clipping in mixture
+    mixture_audio = np.single(augmented_target_audio) * mixture['target_snr_gain'] + \
+                    np.single(noise_segment) * mixture['noise_snr_gain']
+
+    if any(abs(mixture_audio) >= 32768):
+        # Clipping occurred; lower gains to bring audio within int16 bounds
+        gain_adjustment = 32760 / max(abs(mixture_audio))
+        mixture['target_snr_gain'] *= gain_adjustment
+        mixture['noise_snr_gain'] *= gain_adjustment
+        target_gain *= gain_adjustment
+
+    if target_gain == 0:
+        truth = np.zeros((config['num_classes'], len(augmented_target_audio)), dtype=np.int16)
+    else:
+        augmented_target_audio = np.int16(
+            np.single(augmented_target_audio) * mixture['target_snr_gain'])
+        truth = generate_truth(
+            audio=np.int16(np.single(augmented_target_audio) / target_gain),
+            function=target['truth_function'],
+            config=truth_config)
+
+    mixture['class_count'] = get_class_count(
+        truth_index=target['truth_index'],
+        truth=truth,
+        class_weights_threshold=class_weights_threshold)
+
+    mixture['samples'] = len(augmented_target_audio)
+    mixture['target_gain'] = target_gain
+
+    return mixture
+
+
 def main():
-    args = docopt(trim_docstring(__doc__), version=sonusai.version(), options_first=True)
-
-    for config_file in args['CONFIG']:
-        logger.info('Creating mixture database for {}'.format(config_file))
-        config = get_config_from_file(config_file)
-
-        base_name = get_base_name(config, config_file)
-        log_name = base_name + '.log'
-        output_name = base_name + '.json'
-
-        create_file_handler(log_name)
-
-        mixdb = genmixdb(config=config, verbose=args['--verbose'], show_progress=True)
-
-        with open(output_name, mode='w') as file:
-            json.dump(mixdb, file, indent=2)
-            logger.info('Wrote mixture database for {} to {}'.format(config_file, output_name))
-
-
-if __name__ == '__main__':
     try:
-        main()
+        args = docopt(trim_docstring(__doc__), version=sonusai.version(), options_first=True)
+
+        verbose = args['--verbose']
+
+        for config_file in args['CONFIG']:
+            logger.info('Creating mixture database for {}'.format(config_file))
+            config = get_config_from_file(config_file)
+
+            base_name = get_base_name(config, config_file)
+            output_name = base_name + '.json'
+
+            log_name = base_name + '.log'
+            create_file_handler(log_name)
+            update_console_handler(verbose)
+            initial_log_messages('genmixdb')
+
+            mixdb = genmixdb(config=config, show_progress=True)
+
+            with open(output_name, mode='w') as file:
+                json.dump(mixdb, file, indent=2)
+                logger.info('Wrote mixture database for {} to {}'.format(config_file, output_name))
+
     except KeyboardInterrupt:
         logger.info('Canceled due to keyboard interrupt')
         exit()
+
+
+if __name__ == '__main__':
+    main()
