@@ -1,4 +1,3 @@
-#! /usr/bin/env python3
 """gentcst
 
 usage: gentcst [-hvru] [-o OUTPUT] [-f FOLD] [-y ONTOLOGY]
@@ -355,27 +354,38 @@ def get_leaves_from_files(files: list) -> list:
 
 
 def get_labels_from_files(files: list) -> list:
-    label_list = []
-    for item in files:
-        for c in item['labels']:
-            label_list.append(c)
+    all_labels = [file['labels'] for file in files]
 
-    # Converting to set and back to list ensures uniqueness
-    label_list = sorted(list(set(label_list)))
+    # Get labels by depth
+    labels_by_depth = [[] for n in range(max([len(label) for label in all_labels]))]
+    for label in all_labels:
+        for index, name in enumerate(label):
+            labels_by_depth[index].append(name)
 
+    for n in range(len(labels_by_depth)):
+        # Converting to set and back to list ensures uniqueness
+        labels_by_depth[n] = sorted(list(set(labels_by_depth[n])))
+
+    # We want the deepest leaves first
+    labels_by_depth.reverse()
+
+    # Now flatten the list
+    labels_by_depth = [item for sublist in labels_by_depth for item in sublist]
+
+    # Generate index, name pairs
     labels = []
-    for index, item in enumerate(label_list):
-        labels.append({'index': index + 1, 'display_name': item})
+    for index, file in enumerate(labels_by_depth):
+        labels.append({'index': index + 1, 'display_name': file})
 
     return labels
 
 
 def gen_truth_indices(files: list, labels: list):
-    for item in files:
-        item['truth_index'] = []
-        for label in item['labels']:
-            item['truth_index'].append(get_index_from_label(labels, label))
-        item['truth_index'] = sorted(item['truth_index'])
+    for file in files:
+        file['truth_index'] = []
+        for label in file['labels']:
+            file['truth_index'].append(get_index_from_label(labels, label))
+        file['truth_index'] = sorted(file['truth_index'])
 
 
 def get_index_from_label(labels: list, label: str) -> int:
@@ -491,45 +501,46 @@ def report_leaf_fold_data_usage(all_files: list, use_files: list):
 
 
 def main():
-    args = docopt(trim_docstring(__doc__), version=sonusai.version(), options_first=True)
-
-    verbose = args['--verbose']
-    output_name = args['--output']
-    fold = args['--fold']
-    ontology = args['--ontology']
-    hierarchical = args['--hierarchical']
-    update = args['--update']
-
-    if not output_name:
-        output_name = basename(getcwd()) + '.yml'
-
-    log_name = 'gentcst.log'
-
-    create_file_handler(log_name)
-
-    config, labels = gentcst(fold=fold,
-                             ontology=ontology,
-                             hierarchical=hierarchical,
-                             update=update,
-                             verbose=verbose)
-
-    with open(output_name, 'w') as yaml_file:
-        yaml.dump(config, yaml_file)
-        logger.info('Wrote config to {}'.format(output_name))
-
-    csv_fields = ['index', 'display_name']
-    csv_name = splitext(output_name)[0] + '.csv'
-
-    with open(csv_name, 'w') as csv_file:
-        writer = csv.DictWriter(csv_file, fieldnames=csv_fields)
-        writer.writeheader()
-        writer.writerows(labels)
-        logger.info('Wrote labels to {}'.format(csv_name))
-
-
-if __name__ == '__main__':
     try:
-        main()
+        args = docopt(trim_docstring(__doc__), version=sonusai.version(), options_first=True)
+
+        verbose = args['--verbose']
+        output_name = args['--output']
+        fold = args['--fold']
+        ontology = args['--ontology']
+        hierarchical = args['--hierarchical']
+        update = args['--update']
+
+        if not output_name:
+            output_name = basename(getcwd()) + '.yml'
+
+        log_name = 'gentcst.log'
+
+        create_file_handler(log_name)
+
+        config, labels = gentcst(fold=fold,
+                                 ontology=ontology,
+                                 hierarchical=hierarchical,
+                                 update=update,
+                                 verbose=verbose)
+
+        with open(output_name, 'w') as yaml_file:
+            yaml.dump(config, yaml_file)
+            logger.info('Wrote config to {}'.format(output_name))
+
+        csv_fields = ['index', 'display_name']
+        csv_name = splitext(output_name)[0] + '.csv'
+
+        with open(csv_name, 'w') as csv_file:
+            writer = csv.DictWriter(csv_file, fieldnames=csv_fields)
+            writer.writeheader()
+            writer.writerows(labels)
+            logger.info('Wrote labels to {}'.format(csv_name))
+
     except KeyboardInterrupt:
         logger.info('Canceled due to keyboard interrupt')
         exit()
+
+
+if __name__ == '__main__':
+    main()
