@@ -25,6 +25,12 @@ from sonusai.mixture.audio_db import build_target_audio_db  # noqa: E402
 from sonusai.mixture.audio_db import get_noise_audio_from_db  # noqa: E402
 from sonusai.mixture.audio_db import get_target_audio_from_db  # noqa: E402
 from sonusai.mixture.generate_truth import generate_truth  # noqa: E402
+from sonusai.mixture.get_class_weights_threshold import get_class_weights_threshold  # noqa: E402
 from sonusai.mixture.get_config_from_file import get_config_from_file  # noqa: E402
 from sonusai.mixture.get_next_noise import get_next_noise  # noqa: E402
 from sonusai.mixture.get_offsets import get_offsets  # noqa: E402
+from sonusai.mixture.class_count import get_class_count  # noqa: E402
+from sonusai.mixture.class_count import get_total_class_count  # noqa: E402
+from sonusai.mixture.get_mixtures_from_mixid import get_mixtures_from_mixid  # noqa: E402
+from sonusai.mixture.truth_reduction import truth_reduction  # noqa: E402
+from sonusai.mixture.estimate_audio_length import estimate_audio_length  # noqa: E402
