@@ -3,6 +3,7 @@ from pyaaware import ForwardTransform
 from pyaaware import SED
 
 from sonusai import logger
+from sonusai.utils import int16_to_float
 
 
 def strictly_decreasing(list_to_check: list) -> bool:
@@ -46,7 +47,7 @@ def generate_truth(audio: np.ndarray, function: str, config: dict) -> np.ndarray
 
         truth = np.empty((num_classes, 0), dtype=np.single)
         for offset in range(0, len(audio), frame_size):
-            new_truth = sed.execute(fft.energy((audio[offset:offset + frame_size]).astype(np.single) / 32768))
+            new_truth = sed.execute(fft.energy(int16_to_float(audio[offset:offset + frame_size])))
             truth = np.hstack((truth, np.reshape(new_truth, (len(new_truth), 1))))
 
         # This repeats each sample and not the whole sequence
